############################# PSQF 6272 Example 4 using R #####################

# Set width of output and number of significant digits printed,
# number of digits before using scientific notation, shut off significance stars
options(width=120, digits=8, scipen=9, show.signif.stars=FALSE)

#####  Check to see if packages are downloaded, install if not, then load  #####

if (!require("readxl")) install.packages("readxl")
library(readxl) # To import xls or xlsx data as table

if (!require("expss")) install.packages("expss")
library(expss) # To add variable and value labels, sort

if (!require("psych")) install.packages("psych")
library(psych) # To add summary functions

if (!require("lme4")) install.packages("lme4")
library(lme4) # To estimate MLMs using lmer

if (!require("lmerTest")) install.packages("lmerTest")
library(lmerTest) # To get Satterthwaite DDF in lmer

if (!require("performance")) install.packages("performance")
library(performance) # To get ICC using lmer

if (!require("ordinal")) install.packages("ordinal")
library(ordinal) # To estimate MLMs for ordinal outcomes

if (!require("TeachingDemos")) install.packages("TeachingDemos")
library(TeachingDemos) # To create text output files

# Clear workspace (re-run as needed for troubleshooting purposes)
#rm(list = ls())

################################################################################
#####          BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 3             #####
#####              CHANGE "filesave" TO YOUR FOLDER PATH                   #####
################################################################################

# Define variables for working directory and data name -- CHANGE THESE
filesave = "C:\\Dropbox/23_PSQF6272/PSQF6272_Example4/"
filename = "Example4_Data.xlsx"
setwd(dir=filesave)

# Load Jonathan's custom R functions from folder within working directory
functions = paste0("R functions/",dir("R functions/"))
temp = lapply(X=functions, FUN=source)

#  Import trimmed example excel data file from sheet "grade10"
Example4 = read_excel(paste0(filesave,filename), sheet="grade10") 
# Convert to data frame to use in analysis
Example4 = as.data.frame(Example4)

# Open external file to save results to (turn off to use console instead)
txtStart(file=paste0(filesave,"Example4_R_Output.txt"))

# Labels for oriignal variables in comments only
# districtID= "districtID: District ID number"
# studentID=  "studentID: Student ID number"
# schoolID=   "schoolID: School ID number"
# lunch=      "lunch: 0=Paid, 1=Reduced, 2=Free"
# math=       "math: Math Test Score"

print("R Descriptive Statistics within Student-Level Data")
prop.table(table(x=Example4$lunch, useNA="ifany"))

# Create new dummy-coded predictors (start as blank, then recode)
Example4$PvRF=NA; Example4$PRvF=NA 
Example4$PvRF[which(Example4$lunch==0)]=0  # Replace each for paid lunch
Example4$PRvF[which(Example4$lunch==0)]=0
Example4$PvRF[which(Example4$lunch==1)]=1  # Replace each for reduced
Example4$PRvF[which(Example4$lunch==1)]=0
Example4$PvRF[which(Example4$lunch==2)]=1  # Replace each for free lunch
Example4$PRvF[which(Example4$lunch==2)]=1
# PvRF= "PvRF: 0=Paid, 1=Reduced or Free Lunch"
# PRvF= "PRvF: 0=Paid or Reduced, 1=Free Lunch"

# Create indicator variables for demo purposes
Example4$lunch0=NA; Example4$lunch1=NA; Example4$lunch2=NA  
Example4$lunch0[which(Example4$lunch==0)]=1  # Replace each for paid lunch
Example4$lunch1[which(Example4$lunch==0)]=0
Example4$lunch2[which(Example4$lunch==0)]=0
Example4$lunch0[which(Example4$lunch==1)]=0  # Replace each for reduced
Example4$lunch1[which(Example4$lunch==1)]=1
Example4$lunch2[which(Example4$lunch==1)]=0
Example4$lunch0[which(Example4$lunch==2)]=0  # Replace each for free lunch
Example4$lunch1[which(Example4$lunch==2)]=0
Example4$lunch2[which(Example4$lunch==2)]=1
# lunch0= "lunch0: 1=Paid Lunch"
# lunch1= "lunch1: 1=Reduced Lunch"
# lunch2= "lunch2: 1=Free Lunch"

# Filter to only cases complete on all variables to be used below (before cluster means)
Example4 = Example4[complete.cases(Example4[ , c("math","lunch")]),]

# Compute cluster means for level-1 variables using Jonathan's function
Example4 = addUnitMeans(data=Example4, unitVariable="schoolID",
                        meanVariables=c("math","PvRF","PRvF","lunch0","lunch1","lunch2"), 
                        newNames=c("CM_math","CM_PvRF","CM_PRvF","CM_lunch0","CM_lunch1","CM_lunch2"))

print("R Descriptive Statistics within School-Level Data")
schoolMeans = unique(Example4[,c("schoolID","NperschoolID","CM_math","CM_PvRF",
                                 "CM_PRvF","CM_lunch0","CM_lunch1","CM_lunch2")])
describe(x=schoolMeans[ , c("NperschoolID","CM_math","CM_PvRF","CM_PRvF",
                            "CM_lunch0","CM_lunch1","CM_lunch2")])

# Center cluster means so ref school has 0=.70, 1=.11, and 2=.19
Example4$CM_PvRF30 = Example4$CM_PvRF - .30
Example4$CM_PRvF19 = Example4$CM_PRvF - .19
# Cluster-mean-center level-1 predictors for random slopes
Example4$WC_PvRF = Example4$PvRF - Example4$CM_PvRF
Example4$WC_PRvF = Example4$PRvF - Example4$CM_PRvF
# WC_PvRF= "WC_PvRF: Within-Cluster Paid vs Reduced/Free Lunch"
# WC_PRvF= "WC_PRvF: Within-Cluster Paid/Reduced vs Free Lunch"

print("R Descriptive Statistics within Student-Level Data")
describe(x=Example4[ , c("WC_PvRF","WC_PRvF")])


################################################################################
#####                 BEGIN MODELS FOR EXAMPLE 4                           #####
################################################################################

# Empty models

print("R Model 1: Empty Means, Random Intercept for Math")
Model1 = lmer(data=Example4, REML=TRUE, formula=math~1+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model1, chkREML=FALSE); summary(Model1, ddf="Satterthwaite")
print("Show intraclass correlation and its LRT")
icc(Model1); ranova(Model1)

txtComment(" ") # insert blank space in output


print("R: Empty Means, Random Intercept for Ordinal Lunch using ordinal package")
EmptyLunch = clmm(data=Example4, link="logit", formula=as.factor(lunch)~1+(1|schoolID))
print("Show results with -2LL, provides submodel thresholds (multiply by -1 to get intercepts)")
-2*logLik(EmptyLunch); summary(EmptyLunch)
print("ICC using pi^2/3 = 3.29 as L1 residual variance"); icc(EmptyLunch)

print("R: Single-level empty model predicting Ordinal Lunch ignoring school")
SingleLunch = clm(data=Example4, link="logit", formula=as.factor(lunch)~1)
print("Likelihood Ratio Test for Addition of Random Intercept Variance")
DevTest=-2*(logLik(SingleLunch)-logLik(EmptyLunch))
Pvalue=pchisq((DevTest), df=1, lower.tail=FALSE)
print("Test Statistic and P-values for DF=1"); DevTest; Pvalue

txtComment(" ") # insert blank space in output


##################################################################################

# Add Paid vs Reduced/Free Lunch predictor slopes

print("R Model 2a: Add L1 Binary Student Paid vs Reduced/Free Lunch")
Model2a = lmer(data=Example4, REML=TRUE, formula=math~1+PvRF+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model2a, chkREML=FALSE); summary(Model2a, ddf="Satterthwaite")

print("Psuedo-R2 relative to empty means model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model1, largerModel=Model2a)

txtComment(" ") # insert blank space in output


print("R Model 2b: Add L2 Cluster Mean of Paid vs Reduced/Free Lunch")
Model2b = lmer(data=Example4, REML=TRUE, formula=math~1+PvRF+CM_PvRF30+(1|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model2b, chkREML=FALSE); summary(Model2b, ddf="Satterthwaite")
print("L2 PvRF Between Slope"); contest1D(Model2b, L=c(0,1,1))

print("Psuedo-R2 relative to empty means model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model1, largerModel=Model2b)

txtComment(" ") # insert blank space in output


print("R Model 2c: Add Random Slope for Cluster-MC Paid vs Reduced/Free Lunch")
Model2c = lmer(data=Example4, REML=TRUE, 
               formula=math~1+PvRF+CM_PvRF30+(1+WC_PvRF|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model2c, chkREML=FALSE); summary(Model2c, ddf="Satterthwaite")
print("LRT for random slope"); ranova(Model2c)

txtComment(" ") # insert blank space in output


print("R Model 3a: Add Intra-Variable Cross-Level Interaction")
Model3a = lmer(data=Example4, REML=TRUE, formula=math~1+PvRF+CM_PvRF30 
               +PvRF:CM_PvRF30+(1+WC_PvRF|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model3a, chkREML=FALSE); summary(Model3a, ddf="Satterthwaite")

print("Psuedo-R2 relative to random slope model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model2c, largerModel=Model3a)

txtComment(" ") # insert blank space in output


print("R Model 3b: Add Level-2 Quadratic Interaction")
print("R re-ordered quadratic to come right after main effects")
Model3b = lmer(data=Example4, REML=TRUE, formula=math~1+PvRF+CM_PvRF30 
               +I(CM_PvRF30^2) +PvRF:CM_PvRF30 +(1+WC_PvRF|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model3b, chkREML=FALSE); summary(Model3b, ddf="Satterthwaite")
print("Simple L2 PvRF Between Slope"); contest1D(Model3b, L=c(0,1,1,0,0))
print("L2 PvRF Between*CM_PvRF30");    contest1D(Model3b, L=c(0,0,0,1,1))

print("Psuedo-R2 relative to random slope model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model2c, largerModel=Model3b)

print("Total-R2 relative to empty means model using Jonathan's function")
totalRSquaredinator(data=Example4, dvName="math", model=Model3b)

txtComment(" ") # insert blank space in output


######################################################################################

# Add Paid/Reduced vs Free Lunch predictor slopes;

print("R Model 4a: Add 2 Main Effect Slopes for Paid/Reduced vs Free Lunch")
print("R re-ordered new main effects and quadratic before interaction")
Model4a = lmer(data=Example4, REML=TRUE, formula=math~1+PvRF+CM_PvRF30+PRvF+CM_PRvF19
               +I(CM_PvRF30^2) +PvRF:CM_PvRF30 +(1+WC_PvRF|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model4a, chkREML=FALSE); summary(Model4a, ddf="Satterthwaite")
print("Simple L2 PvRF Between Slope"); contest1D(Model4a, L=c(0,1,1,0,0,0,0))
print("L2 PvRF Between*CM_PvRF30");    contest1D(Model4a, L=c(0,0,0,0,0,1,1))
print("L2 PRvF Between Slope");        contest1D(Model4a, L=c(0,0,0,1,1,0,0))

print("Psuedo-R2 relative to random slope model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model2c, largerModel=Model4a)

print("Total-R2 relative to empty means model using Jonathan's function")
totalRSquaredinator(data=Example4, dvName="math", model=Model4a)

txtComment(" ") # insert blank space in output


print("R Model 4b: Add Random Slope for Cluster-MC Paid/Reduced vs Free Lunch")
print("R re-ordered new main effects and quadratic before interaction")
Model4b = lmer(data=Example4, REML=TRUE, formula=math~1+PvRF+CM_PvRF30+PRvF+CM_PRvF19
               +I(CM_PvRF30^2) +PvRF:CM_PvRF30 +(1+WC_PvRF+WC_PRvF|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model4b, chkREML=FALSE); summary(Model4b, ddf="Satterthwaite")
print("LRT for second random slope"); ranova(Model4b)

txtComment(" ") # insert blank space in output


print("R Model 5a: Add 2 Interaction Slopes for Paid/Reduced vs Free Lunch")
print("R re-ordered all main effects and quadratics before interactions")
Model5a = lmer(data=Example4, REML=TRUE, formula=math~1+PvRF+CM_PvRF30+PRvF+CM_PRvF19
               +I(CM_PvRF30^2) +I(CM_PRvF19^2) +PvRF:CM_PvRF30 +PRvF:CM_PRvF19 
               +(1+WC_PvRF|schoolID))
print("Show results using Satterthwaite DDF including -2LL as deviance")
llikAIC(Model5a, chkREML=FALSE); summary(Model5a, ddf="Satterthwaite")
print("Simple L2 PvRF Between");    contest1D(Model5a, L=c(0,1,1,0,0,0,0,0,0))
print("L2 PvRF Between*CM_PvRF30"); contest1D(Model5a, L=c(0,0,0,0,0,1,0,1,0))
print("Simple PRvF L2 Between");    contest1D(Model5a, L=c(0,0,0,1,1,0,0,0,0))
print("L2 PRvF Between*CM_PRvF19"); contest1D(Model5a, L=c(0,0,0,0,0,0,1,0,1))

print("Psuedo-R2 relative to single random slope model using Jonathan's function")
pseudoRSquaredinator(smallerModel=Model2c, largerModel=Model5a)

print("Total-R2 relative to empty means model using Jonathan's function")
totalRSquaredinator(data=Example4, dvName="math", model=Model5a)

txtComment(" ") # insert blank space in output


#######################################################################################
# Close external file of saved results
txtStop()



