// Prevent "more" messages from appearing
   set more off
// Control line length
   set linesize 150

///////////////////////////////////////////////////////////////////////////////////////
////////               BEGIN DATA IMPORT AND PREPARATION FOR EXAMPLE 4          ///////
////////                  CHANGE "filesave" TO YOUR FOLDER PATH                 ///////
///////////////////////////////////////////////////////////////////////////////////////

// Define global variable for file location to be replaced in code below
// \\Client\ precedes path in Virtual Desktop outside H drive
   global filesave "C:\Dropbox\23_PSQF6272\PSQF6272_Example4"

// Open trimmed example excel data file from sheet "grade10" and clear away existing data
   clear // clear memory in case of open data
   import excel "$filesave\Example4_Data.xlsx", firstrow case(preserve) sheet("grade10") clear 
   
// Open external file to save results to
   log using $filesave\Example4_STATA_Output.log, replace   

// Add labels to original variables
   label variable districtID "districtID: District ID number"
   label variable studentID  "studentID: Student ID number"
   label variable schoolID   "schoolID: School ID number"
   label variable lunch      "lunch: 0=Paid, 1=Reduced, 2=Free"
   label variable math       "math: Math Test Score"

display "STATA Descriptive Statistics within Student-Level Data"
tabulate lunch
       
// Create new dummy-coded predictors (start as blank, then recode)
   gen PvRF=.  
   gen PRvF=.
   replace PvRF=0 if lunch==0  // Replace each for paid lunch
   replace PRvF=0 if lunch==0
   replace PvRF=1 if lunch==1  // Replace each for reduced lunch
   replace PRvF=0 if lunch==1
   replace PvRF=1 if lunch==2  // Replace each for free lunch
   replace PRvF=1 if lunch==2
   label variable PvRF "PvRF: 0=Paid, 1=Reduced or Free Lunch"
   label variable PRvF "PRvF: 0=Paid or Reduced, 1=Free Lunch"
// Create indicator variables for demo purposes
   gen lunch0=.
   gen lunch1=.
   gen lunch2=.
   replace lunch0=1 if lunch==0  // Replace each for paid lunch
   replace lunch1=0 if lunch==0
   replace lunch2=0 if lunch==0
   replace lunch0=0 if lunch==1  // Replace each for reduced lunch
   replace lunch1=1 if lunch==1
   replace lunch2=0 if lunch==1
   replace lunch0=0 if lunch==2  // Replace each for free lunch
   replace lunch1=0 if lunch==2
   replace lunch2=1 if lunch==2
   label variable lunch0 "lunch0: 1=Paid Lunch"
   label variable lunch1 "lunch1: 1=Reduced Lunch"
   label variable lunch2 "lunch2: 1=Free Lunch"
     
// Filter to complete cases before computing cluster means
   egen nmiss=rowmiss(math lunch)
   drop if nmiss>0
       
// Compute cluster means for level-1 variables
   sort schoolID
   egen schoolN = count(math),    by(schoolID)
   egen CM_math = mean(math),     by(schoolID)     
   egen CM_PvRF = mean(PvRF),     by(schoolID)     
   egen CM_PRvF = mean(PRvF),     by(schoolID) 
   egen CM_lunch0 = mean(lunch0), by(schoolID)
   egen CM_lunch1 = mean(lunch1), by(schoolID)
   egen CM_lunch2 = mean(lunch2), by(schoolID)
     
display "STATA Descriptive Statistics within School-Level Data"
preserve  // Save for later use, then compute school-level dataset
collapse  schoolN CM_math CM_PvRF CM_PRvF CM_lunch0 CM_lunch1 CM_lunch2, by(schoolID)
format    schoolN CM_math CM_PvRF CM_PRvF CM_lunch0 CM_lunch1 CM_lunch2  %4.2f
summarize schoolN CM_math CM_PvRF CM_PRvF CM_lunch0 CM_lunch1 CM_lunch2, format

// Go back to student-level dataset
   restore   
// Center cluster means so ref school has 0=.70, 1=.11, and 2=.19
   gen CM_PvRF30 = CM_PvRF - .30
   gen CM_PRvF19 = CM_PRvF - .19
// Cluster-mean-center level-1 predictors for random slopes
   gen WC_PvRF = PvRF - CM_PvRF
   gen WC_PRvF = PRvF - CM_PRvF
   label variable WC_PvRF "WC_PvRF: Within-Cluster Paid vs Reduced/Free Lunch"
   label variable WC_PRvF "WC_PRvF: Within-Cluster Paid/Reduced vs Free Lunch"

display "STATA Descriptive Statistics within Student-Level Data"
summarize WC_PvRF WC_PRvF, detail


///////////////////////////////////////////////////////////////////////////////////////
////////                     BEGIN MODELS FOR EXAMPLE 4                          //////
///////////////////////////////////////////////////////////////////////////////////////

// Empty models

display "STATA Model 1: Empty Means, Random Intercept for Math"
mixed math , || schoolID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat icc                       // Intraclass correlation

display "STATA: Empty Means, Random Intercept for Ordinal Lunch"
display "Provides submodel thresholds (multiply by -1 to get intercepts)"
meologit lunch , || schoolID: , nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
display "ICC = " 1.793843/(1.793843+3.29)  
          
// Add Paid vs Reduced/Free Lunch predictor slopes           
                
display "STATA Model 2a: Add L1 Binary Student Paid vs Reduced/Free Lunch"
mixed math c.PvRF, || schoolID: , reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  

display "STATA Model 2b: Add L2 Cluster Mean of Paid vs Reduced/Free Lunch"
mixed math c.PvRF c.CM_PvRF30, || schoolID: , ///
           reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2        // Print -2LL for model  
lincom c.PvRF*1 + c.CM_PvRF30*1   // L2 PvRF Between Slope
estimates store Fix               // Save fit for LRT

display "STATA Model 2c: Add Random Slope for Cluster-MC Paid vs Reduced/Free Lunch"
mixed math c.PvRF c.CM_PvRF30, || schoolID: WC_PvRF, ///
           cov(un) reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  
estat recovariance, relevel(schoolID) correlation  // Random effect correlations
estimates store Rand            // Save fit for LRT
lrtest Rand Fix                 // LRT for random slope
 
display "STATA Model 3a: Add Intra-Variable Cross-Level Interaction"
mixed math c.PvRF c.CM_PvRF30 c.PvRF#c.CM_PvRF30, || schoolID: WC_PvRF, ///
           cov(un) reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2      // Print -2LL for model  

display "STATA Model 3b: Add Level-2 Quadratic Interaction"
mixed math c.PvRF c.CM_PvRF30 c.PvRF#c.CM_PvRF30 c.CM_PvRF30#c.CM_PvRF30, ///
           || schoolID: WC_PvRF, cov(un) reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2        // Print -2LL for model  
estat recovariance, relevel(schoolID) correlation  // Random effect correlations
lincom c.PvRF*1 + c.CM_PvRF30*1                          // Simple L2 PvRF Between Slope
lincom c.PvRF#c.CM_PvRF30*1 + c.CM_PvRF30#c.CM_PvRF30*1  // L2 PvRF Between*CM_PvRF30
predict pred3b
corr math pred3b                // Get total r to make R2
display "Total-R2 = " r(rho)^2  // Print total R2 relative to empty model   

// Add Paid/Reduced vs Free Lunch predictor slopes

display "STATA Model 4a: Add 2 Main Effect Slopes for Paid/Reduced vs Free Lunch"
mixed math c.PvRF c.CM_PvRF30 c.PvRF#c.CM_PvRF30 c.CM_PvRF30#c.CM_PvRF30 ///
           c.PRvF c.CM_PRvF19, || schoolID: WC_PvRF, ///
           cov(un) reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2       // Print -2LL for model 
estat recovariance, relevel(schoolID) correlation  // Random effect correlations
lincom c.PvRF*1 + c.CM_PvRF30*1                          // Simple L2 PvRF Between Slope
lincom c.PvRF#c.CM_PvRF30*1 + c.CM_PvRF30#c.CM_PvRF30*1  // L2 PvRF Between*CM_PvRF30
lincom c.PRvF*1 + c.CM_PRvF19*1                          // L2 PRvF Between Slope
estimates store Fix2             // Save fit for LRT 

display "STATA Model 4b: Add Random Slope for Cluster-MC Paid/Reduced vs Free Lunch"
mixed math c.PvRF c.CM_PvRF30 c.PvRF#c.CM_PvRF30 c.CM_PvRF30#c.CM_PvRF30 ///
           c.PRvF c.CM_PRvF19, || schoolID: WC_PvRF WC_PRvF, ///
           cov(un) reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2        // Print -2LL for model  
estat recovariance, relevel(schoolID) correlation  // Random effect correlations
estimates store Rand2             // Save fit for LRT
*lrtest Rand2 Fix2                // LRT for second random slope --> broken here!

display "STATA Model 5a: Add 2 Interaction Slopes for Paid/Reduced vs Free Lunch"
mixed math c.PvRF c.CM_PvRF30 c.PvRF#c.CM_PvRF30 c.CM_PvRF30#c.CM_PvRF30  ///
           c.PRvF c.CM_PRvF19 c.PRvF#c.CM_PRvF19 c.CM_PRvF19#c.CM_PRvF19, ///
           || schoolID: WC_PvRF, /// Only PvRF random slope, PRvF is systematically varying
           cov(un) reml dfmethod(satterthwaite) dftable(pvalue) nolog
display "-2LL = " e(ll)*-2        // Print -2LL for model  
estat recovariance, relevel(schoolID) correlation  // Random effect correlations
lincom c.PvRF*1 + c.CM_PvRF30*1                          // Simple L2 PvRF Between 
lincom c.PvRF#c.CM_PvRF30*1 + c.CM_PvRF30#c.CM_PvRF30*1  // L2 PvRF Between*CM_PvRF30
lincom c.PRvF*1 + c.CM_PRvF19*1                          // Simple L2 PRvF Between
lincom c.PRvF#c.CM_PRvF19*1 + c.CM_PRvF19#c.CM_PRvF19*1  // L2 PRvF Between*CM_PRvF19
predict pred5a
corr math pred5a                // Get total r to make R2
display "Total-R2 = " r(rho)^2  // Print total R2 relative to empty model  


// Close external file of saved results
   log close

